@@ -15,7 +15,234 @@ Common functions for converting types to and from XML files
 """
 
 
-def convert_xml_attribute(stringattribute, possible_types, constants=None, logger=None, list_return=False):
+def convert_to_xml(value, schema_dict, name, text=False, logger=None, list_return=False):
+    """
+    Tries to converts a given string to the types specified in the schema_dict.
+    First succeeded conversion will be returned
+
+    If no logger is given and a attribute cannot be converted an error is raised
+
+    :param stringattribute: str, Attribute to convert.
+    :param schema_dict: Schema dictionary containing all the information
+    :param name: name of the attribute or element
+    :param text: bool, decides whether to take the definitions for text or attributes
+    :param constants: dict, of constants defined in fleur input
+    :param logger: logger object for logging warnings
+                   if given the errors are logged and the list is returned with the unconverted values
+                   otherwise a error is raised, when the first conversion fails
+    :param list_return: if True, the returned quantity is always a list even if only one element is in it
+
+    :return: The converted value of the first successful conversion
+    """
+
+    if text:
+        if name not in schema_dict['text_types']:
+            raise KeyError(f'Unknown text tag: {name}')
+        definitions = schema_dict['text_types'][name]
+        float_format = '16.13'
+    else:
+        if name not in schema_dict['attrib_types']:
+            raise KeyError(f'Unknown attibute name: {name}')
+        definitions = schema_dict['attrib_types'][name]
+        float_format = '.10'
+
+    return convert_to_xml_explicit(value,
+                                   definitions,
+                                   logger=logger,
+                                   list_return=list_return,
+                                   float_format=float_format)
+
+
+def convert_from_xml(xmlstring, schema_dict, name, text=False, constants=None, logger=None, list_return=False):
+    """
+    Tries to converts a given string to the types specified in the schema_dict.
+    First succeeded conversion will be returned
+
+    If no logger is given and a attribute cannot be converted an error is raised
+
+    :param stringattribute: str, Attribute to convert.
+    :param schema_dict: Schema dictionary containing all the information
+    :param name: name of the attribute or element
+    :param text: bool, decides whether to take the definitions for text or attributes
+    :param constants: dict, of constants defined in fleur input
+    :param logger: logger object for logging warnings
+                   if given the errors are logged and the list is returned with the unconverted values
+                   otherwise a error is raised, when the first conversion fails
+    :param list_return: if True, the returned quantity is always a list even if only one element is in it
+
+    :return: The converted value of the first successful conversion
+    """
+
+    if not isinstance(xmlstring, list):
+        xmlstring = [xmlstring]
+
+    if text:
+        if name not in schema_dict['text_types']:
+            raise KeyError(f'Unknown text tag: {name}')
+        definitions = schema_dict['text_types'][name]
+        xmlstring = [string.strip() for string in xmlstring]
+    else:
+        if name not in schema_dict['attrib_types']:
+            raise KeyError(f'Unknown attibute name: {name}')
+        definitions = schema_dict['attrib_types'][name]
+
+    return convert_from_xml_explicit(xmlstring,
+                                     definitions,
+                                     constants=constants,
+                                     logger=logger,
+                                     list_return=list_return)
+
+
+def convert_from_xml_explicit(xmlstring, definitions, constants=None, logger=None, list_return=False):
+    """
+    Tries to converts a given string to the types given in definitions.
+    First succeeded conversion will be returned
+
+    If no logger is given and a attribute cannot be converted an error is raised
+
+    :param stringattribute: str, Attribute to convert.
+    :param definitions: list of :py:class:`~masci_tools.io.parsers.fleur.fleur_schema.AttributeType` definitions
+    :param constants: dict, of constants defined in fleur input
+    :param logger: logger object for logging warnings
+                   if given the errors are logged and the list is returned with the unconverted values
+                   otherwise a error is raised, when the first conversion fails
+    :param list_return: if True, the returned quantity is always a list even if only one element is in it
+
+    :return: The converted value of the first successful conversion
+    """
+
+    if not isinstance(xmlstring, list):
+        xmlstring = [xmlstring]
+
+    converted_list = []
+    all_success = True
+    for text in xmlstring:
+
+        split_text = text.split(' ')
+        while '' in split_text:
+            split_text.remove('')
+
+        text_definitions = []
+        for definition in definitions:
+            if definition.length == len(split_text):
+                text_definitions.append(definition)
+
+        if not text_definitions:
+            for definition in definitions:
+                if definition.length in ('unbounded', 1):
+                    text_definitions.append(definition)
+
+        if not text_definitions:
+            if logger is None:
+                raise ValueError(f"Could not convert '{text}', no matching definition found")
+            else:
+                logger.warning("Could not convert '%s', no matching definition found", text)
+            converted_list.append(text)
+            all_success = False
+            continue
+
+        types = tuple(definition.base_type for definition in text_definitions)
+        lengths = {definition.length for definition in text_definitions}
+
+        if len(text_definitions) == 1:
+            if text_definitions[0].length == 1:
+                split_text = [text]
+
+        converted_text, suc = convert_from_xml_single_values(split_text,
+                                                             types,
+                                                             constants=constants,
+                                                             logger=logger,
+                                                             list_return=True)
+
+        all_success = all_success and suc
+
+        if len(converted_text) == 1 and 'unbounded' not in lengths:
+            converted_text = converted_text[0]
+        elif len(converted_text) == 0 and 'unbounded' not in lengths:
+            converted_text = ''
+
+        converted_list.append(converted_text)
+
+    ret_value = converted_list
+    if len(converted_list) == 1 and not list_return:
+        ret_value = converted_list[0]
+
+    return ret_value, all_success
+
+
+def convert_to_xml_explicit(value, definitions, logger=None, float_format='.10', list_return=False):
+    """
+    Tries to convert a given list of values to str for a xml file based on the definitions (length and type).
+    First succeeded conversion will be returned
+
+    :param textvalue: value to convert
+    :param definitions: list of :py:class:`~masci_tools.io.parsers.fleur.fleur_schema.AttributeType` definitions
+    :param logger: logger object for logging warnings
+                   if given the errors are logged and the list is returned with the unconverted values
+                   otherwise a error is raised, when the first conversion fails
+    :param list_return: if True, the returned quantity is always a list even if only one element is in it
+
+    :return: The converted value of the first succesful conversion
+    """
+    import numpy as np
+
+    lengths = {definition.length for definition in definitions}
+
+    if not isinstance(value, (list, np.ndarray)):
+        value = [value]
+    elif not isinstance(value[0], (list, np.ndarray)) and lengths != {1}:
+        value = [value]
+
+    converted_list = []
+    all_success = True
+    for val in value:
+
+        if not isinstance(val, (list, np.ndarray)):
+            val = [val]
+
+        text_definitions = []
+        for definition in definitions:
+            if definition.length == len(val):
+                text_definitions.append(definition)
+
+        if not text_definitions:
+            for definition in definitions:
+                if definition.length == 'unbounded':
+                    text_definitions.append(definition)
+
+        if not text_definitions:
+            if len(val) == 1 and isinstance(val[0], str):
+                converted_list.append(val[0])
+                continue
+
+            if logger is None:
+                raise ValueError(f"Could not convert '{val}', no matching definition found")
+            else:
+                logger.warning("Could not convert '%s', no matching definition found", val)
+
+            converted_list.append('')
+            all_success = False
+            continue
+
+        types = tuple(definition.base_type for definition in text_definitions)
+
+        converted_text, suc = convert_to_xml_single_values(val,
+                                                           types,
+                                                           logger=logger,
+                                                           float_format=float_format,
+                                                           list_return=True)
+        all_success = all_success and suc
+
+        converted_list.append(' '.join(converted_text))
+
+    ret_value = converted_list
+    if len(converted_list) == 1 and not list_return:
+        ret_value = converted_list[0]
+
+    return ret_value, all_success
+
+
+def convert_from_xml_single_values(xmlstring, possible_types, constants=None, logger=None, list_return=False):
     """
     Tries to converts a given string attribute to the types given in possible_types.
     First succeeded conversion will be returned
@@ -32,65 +259,69 @@ def convert_xml_attribute(stringattribute, possible_types, constants=None, logge
 
     :return: The converted value of the first successful conversion
     """
-    from masci_tools.util.fleur_calculate_expression import calculate_expression
+    from masci_tools.util.fleur_calculate_expression import calculate_expression, MissingConstant
 
-    if not isinstance(stringattribute, list):
-        stringattribute = [stringattribute]
-
-    exceptions = []
+    if not isinstance(xmlstring, list):
+        xmlstring = [xmlstring]
 
     converted_list = []
     all_success = True
-    for attrib in stringattribute:
+    for text in xmlstring:
+
+        exceptions = []
         for value_type in possible_types:
             if value_type == 'float':
                 try:
-                    converted_value = float(attrib)
+                    converted_value = float(text)
                 except (ValueError, TypeError) as exc:
                     exceptions.append(exc)
                     continue
 
             elif value_type == 'float_expression':
-                if constants is None:
-                    raise ValueError(
-                        "For calculating attributes of the type 'float_expression' constants have to be given")
                 try:
-                    converted_value = calculate_expression(attrib, constants)
+                    converted_value = calculate_expression(text, constants=constants)
                 except ValueError as exc:
                     exceptions.append(exc)
                     continue
+                except MissingConstant as exc:
+                    new_exc = MissingConstant(f'No value available for expression {exc}\n'
+                                              'Please provide the value for this constant'
+                                              ' by using the read_constants function for example')
+
+                    exceptions.append(new_exc)
+                    continue
 
             elif value_type == 'int':
                 try:
-                    converted_value = int(attrib)
+                    converted_value = int(text)
                 except (ValueError, TypeError) as exc:
                     exceptions.append(exc)
                     continue
 
             elif value_type == 'switch':
                 try:
-                    converted_value = convert_from_fortran_bool(attrib)
+                    converted_value = convert_from_fortran_bool(text)
                 except (ValueError, TypeError) as exc:
                     exceptions.append(exc)
                     continue
 
             elif value_type == 'string':
-                converted_value = str(attrib)
+                converted_value = str(text)
 
             converted_list.append(converted_value)
             break
         else:
             if logger is None:
-                raise ValueError(f"Could not convert '{attrib}'. Tried: {possible_types}.\n"
+                raise ValueError(f"Could not convert '{text}'. Tried: {possible_types}.\n"
                                  'The following errors occurred:\n   ' + '\n   '.join([str(exc) for exc in exceptions]))
             else:
-                logger.warning("Could not convert '%s'. The following errors occurred:", attrib)
+                logger.warning("Could not convert '%s'. The following errors occurred:", text)
 
                 for exc in exceptions:
                     logger.warning('   %s', str(exc))
                     logger.debug(exc, exc_info=exc)
 
-            converted_list.append(attrib)
+            converted_list.append(text)
             all_success = False
 
     ret_value = converted_list
@@ -100,13 +331,13 @@ def convert_xml_attribute(stringattribute, possible_types, constants=None, logge
     return ret_value, all_success
 
 
-def convert_attribute_to_xml(attributevalue, possible_types, logger=None, float_format='.10', list_return=False):
+def convert_to_xml_single_values(value, possible_types, logger=None, float_format='.10', list_return=False):
     """
     Tries to converts a given attributevalue to a string for a xml file according
     to the types given in possible_types.
     First succeeded conversion will be returned
 
-    :param attributevalue: value to convert.
+    :param value: value to convert.
     :param possible_types: list of str What types it will try to convert from
     :param logger: logger object for logging warnings
                    if given the errors are logged and the list is returned with the unconverted values
@@ -117,57 +348,49 @@ def convert_attribute_to_xml(attributevalue, possible_types, logger=None, float_
     """
     import numpy as np
 
-    if not isinstance(attributevalue, (list, np.ndarray)):
-        attributevalue = [attributevalue]
+    if not isinstance(value, (list, np.ndarray)):
+        value = [value]
 
-    possible_types = possible_types.copy()
-
-    if 'int' in possible_types:  #Since it just converts to string
-        possible_types.remove('int')
-        possible_types.append('int')
-
-    if 'string' in possible_types:  #Move string to back
-        possible_types.remove('string')
-
-    possible_types.append('string')  #Always try string
+    if 'string' not in possible_types:
+        possible_types = possible_types + ('string',)  #Always try string
 
     converted_list = []
     exceptions = []
     all_success = True
-    for value in attributevalue:
+    for val in value:
 
         for value_type in possible_types:
             if value_type in ('float', 'float_expression'):
                 try:
-                    converted_value = f'{value:{float_format}f}'
+                    converted_value = f'{val:{float_format}f}'
                 except ValueError as exc:
                     exceptions.append(exc)
                     continue
 
             elif value_type == 'switch':
                 try:
-                    converted_value = convert_to_fortran_bool(value)
+                    converted_value = convert_to_fortran_bool(val)
                 except (ValueError, TypeError) as exc:
                     exceptions.append(exc)
                     continue
 
             elif value_type in ('string', 'int'):
-                converted_value = str(value)
+                converted_value = str(val)
 
             converted_list.append(converted_value)
             break
         else:
             if logger is None:
-                raise ValueError(f"Could not convert '{value}' to text. Tried: {possible_types}.\n"
+                raise ValueError(f"Could not convert '{val}' to text. Tried: {possible_types}.\n"
                                  'The following errors occurred:\n   ' + '\n   '.join([str(exc) for exc in exceptions]))
             else:
-                logger.warning("Could not convert '%s' to text. The following errors occurred:", value)
+                logger.warning("Could not convert '%s' to text. The following errors occurred:", val)
 
                 for exc in exceptions:
                     logger.warning('   %s', str(exc))
                     logger.debug(exc, exc_info=exc)
 
-            converted_list.append(value)
+            converted_list.append(val)
             all_success = False
 
     ret_value = converted_list
@@ -177,154 +400,6 @@ def convert_attribute_to_xml(attributevalue, possible_types, logger=None, float_
     return ret_value, all_success
 
 
-def convert_xml_text(tagtext, possible_definitions, constants=None, logger=None, list_return=False):
-    """
-    Tries to converts a given string text based on the definitions (length and type).
-    First succeeded conversion will be returned
-
-    :param tagtext: str, text to convert.
-    :param possible_defintions: list of dicts What types it will try to convert to
-    :param constants: dict, of constants defined in fleur input
-    :param logger: logger object for logging warnings
-                   if given the errors are logged and the list is returned with the unconverted values
-                   otherwise a error is raised, when the first conversion fails
-    :param list_return: if True, the returned quantity is always a list even if only one element is in it
-
-    :return: The converted value of the first succesful conversion
-    """
-
-    if not isinstance(tagtext, list):
-        tagtext = [tagtext]
-
-    converted_list = []
-    all_success = True
-    for text in tagtext:
-
-        base_text = text.strip()
-        split_text = text.split(' ')
-        while '' in split_text:
-            split_text.remove('')
-
-        text_definition = None
-        for definition in possible_definitions:
-            if definition['length'] == len(split_text):
-                text_definition = definition
-
-        if text_definition is None:
-            for definition in possible_definitions:
-                if definition['length'] == 'unbounded' or \
-                   definition['length'] == 1:
-                    text_definition = definition
-
-        if text_definition is None:
-            if logger is None:
-                raise ValueError(f"Failed to convert '{text}', no matching definition found")
-            else:
-                logger.warning("Failed to convert '%s', no matching definition found", text)
-            converted_list.append(text)
-            all_success = False
-            continue
-
-        #Avoid splitting the text accidentally if there is no length restriction
-        if text_definition['length'] == 1:
-            split_text = [base_text]
-
-        converted_text = []
-        for value in split_text:
-            converted_value, suc = convert_xml_attribute(value,
-                                                         text_definition['type'],
-                                                         constants=constants,
-                                                         logger=logger)
-            converted_text.append(converted_value)
-            if not suc:
-                all_success = False
-
-        if len(converted_text) == 1 and text_definition['length'] != 'unbounded':
-            converted_text = converted_text[0]
-
-        converted_list.append(converted_text)
-
-    ret_value = converted_list
-    if len(converted_list) == 1 and not list_return:
-        ret_value = converted_list[0]
-
-    return ret_value, all_success
-
-
-def convert_text_to_xml(textvalue, possible_definitions, logger=None, float_format='16.13', list_return=False):
-    """
-    Tries to convert a given list of values to str for a xml file based on the definitions (length and type).
-    First succeeded conversion will be returned
-
-    :param textvalue: value to convert
-    :param possible_definitions: list of dicts What types it will try to convert to
-    :param logger: logger object for logging warnings
-                   if given the errors are logged and the list is returned with the unconverted values
-                   otherwise a error is raised, when the first conversion fails
-    :param list_return: if True, the returned quantity is always a list even if only one element is in it
-
-    :return: The converted value of the first succesful conversion
-    """
-    import numpy as np
-
-    if not isinstance(textvalue, (list, np.ndarray)):
-        textvalue = [textvalue]
-    elif not isinstance(textvalue[0], (list, np.ndarray)):
-        textvalue = [textvalue]
-
-    converted_list = []
-    all_success = True
-    for text in textvalue:
-
-        if not isinstance(text, (list, np.ndarray)):
-            text = [text]
-
-        text_definition = None
-        for definition in possible_definitions:
-            if definition['length'] == len(text):
-                text_definition = definition
-
-        if text_definition is None:
-            for definition in possible_definitions:
-                if definition['length'] == 'unbounded':
-                    text_definition = definition
-
-        if text_definition is None:
-            if len(text) == 1 and isinstance(text[0], str):
-                converted_list.append(text[0])
-                continue
-
-            if logger is None:
-                raise ValueError(f"Failed to convert '{text}', no matching definition found")
-            else:
-                logger.warning("Failed to convert '%s', no matching definition found", text)
-
-            converted_list.append('')
-            all_success = False
-            continue
-
-        converted_text = []
-        for value in text:
-            converted_value, suc = convert_attribute_to_xml(value,
-                                                            text_definition['type'],
-                                                            logger=logger,
-                                                            float_format=float_format)
-            converted_text.append(converted_value)
-            if not suc:
-                if isinstance(value, str):
-                    converted_list.append(value)
-                    continue
-                all_success = False
-
-        converted_list.append(' '.join(converted_text))
-
-    ret_value = converted_list
-    if len(converted_list) == 1 and not list_return:
-        ret_value = converted_list[0]
-
-    return ret_value, all_success
-
-
 def convert_from_fortran_bool(stringbool):
     """
     Converts a string in this case ('T', 'F', or 't', 'f') to True or False
